
"""PgQ producer helpers for Python.
"""

import skytools

class Producer:
    """Base class for producers."""
    def __init__(self, name):
        self.name = name
        self.plan = None

    def send_event(self, db, queue_name, ref_id, payload = None):
        """Publish event using DBAPI connection."""

        if payload:
            payload = self.render_payload(payload)

        cx = db.cursor()
        cx.execute("select pgq.insert_event(%s, %s, %s, %s)",
            [self.name, queue_name, ref_id, payload])

    def backend_event(self, queue_name, ref_id, payload = None):
        """Publish event from pl/python.
        
        It assumes that whole class instance is cached in SD or GD.
        """

        ref_id = str(ref_id)
        if payload:
            payload = self.render_payload(payload)

        import plpy

        # get plan
        if not self.plan:
            sql = "select pgq.insert_event($1, $2, $3, $4)"
            types = ["text", "text", "text", "text"]
            self.plan = plpy.prepare(sql, types)

        # run it
        plpy.execute(self.plan, [self.name, queue_name, ref_id, payload])

    def render_payload(self, payload):
        """Format non-text payloads.
        
        Currently dict is published in url-encoded form.
        """

        if type(payload) == type({}):
            payload = skytools.db_urlencode(payload)
        return payload

def bulk_insert_events(curs, rows, fields, queue_name):
    q = "select pgq.current_event_table(%s)"
    curs.execute(q, [queue_name])
    tbl = curs.fetchone()[0]
    skytools.magic_insert(curs, tbl, rows, fields)

