#! /usr/bin/env python

# puts events into queue specified by ev.producer

import sys, os, pgq, skytools

class QueueSplitter(pgq.SerialConsumer):
    def __init__(self, args):
        pgq.SerialConsumer.__init__(self, "queue_splitter", "src_db", "dst_db", args)

    def process_remote_batch(self, batch_id, ev_list, dst_db):
        cache = {}
        for ev in ev_list:
            row = [ev.ref_id, ev.payload, ev.producer, ev.creation_date]
            if ev.producer not in cache:
                cache[ev.producer] = []
            cache[ev.producer].append(row)
            ev.tag_done()

        # should match the composed row
        fields = ['ref_id', 'payload', 'producer', 'creation_date']

        # now send them to right queues
        curs = dst_db.cursor()
        for queue, rows in cache.items():
            pgq.bulk_insert_events(curs, rows, fields, queue)

if __name__ == '__main__':
    script = QueueSplitter(sys.argv[1:])
    script.start()

