create or replace function pgq.event_retry(
    x_batch_id bigint,
    x_event_id bigint,
    x_retry_time timestamptz)
returns integer as $$
begin
    insert into pgq.retry_event (queue_id, consumer_id, batch_id, event_id,
           retry_count, retry_after_date,
           creation_date, ref_id, payload, producer)
    select b.queue_id, b.consumer_id, b.batch_id, ev.event_id,
           coalesce(ev.retry_count, 0) + 1, x_retry_time, ev.creation_date,
           ev.ref_id, ev.payload, ev.producer
      from pgq.fetch_batch_events(x_batch_id) ev,
           pgq.active_batch b
     where b.batch_id = x_batch_id
       and ev.event_id = x_event_id;
    if not found then
        raise exception 'event not found';
    end if;
    return 1;

-- dont worry if the event is already in queue
exception
    when unique_violation then
        return 0;
end;
$$ language plpgsql security definer;

create or replace function pgq.event_retry(
    x_batch_id bigint,
    x_event_id bigint,
    x_retry_seconds integer)
returns integer as $$
declare
    new_retry  timestamptz;
begin
    new_retry := current_timestamp + ((x_retry_seconds || ' seconds')::interval);
    return pgq.event_retry(x_batch_id, x_event_id, new_retry);
end;
$$ language plpgsql security definer;

create or replace function pgq.event_retry_raw(
    x_queue text,
    x_consumer text,
    x_event_id bigint,
    x_ref_id text,
    x_payload text,
    x_producer text,
    x_creation_time timestamptz,
    x_retry_time timestamptz,
    x_retry_count integer)
returns bigint as $$
declare
    cf record;
    id bigint;
begin
    select wm.consumer_id, wm.queue_id, q.event_id_seq into cf
      from pgq.consumer c, pgq.queue_config q,
           pgq.high_watermark wm
     where q.queue_name = x_queue
       and c.consumer_name = x_consumer
       and wm.consumer_id = c.consumer_id
       and wm.queue_id = q.queue_id;
    if not found then
        raise exception 'consumer not registered';
    end if;

    id := x_event_id;
    if id is null then
        id := nextval(cf.event_id_seq);
    end if;

    insert into pgq.retry_event (queue_id, consumer_id, batch_id, event_id,
           retry_count, retry_after_date, creation_date,
           ref_id, payload, producer)
    values (cf.queue_id, cf,consumer_id, null, id,
           x_retry_count, x_retry_time, x_creation_time,
           x_ref_id, x_payload, x_producer);

    return id;
end;
$$ language plpgsql security definer;

