
create or replace function pgq.finish_batch(
    x_batch_id bigint,
    x_total integer,
    x_retry integer,
    x_failed integer)
returns integer as $$
declare
    do_batch_log boolean;
begin
    -- update stats
    perform 1 from pgq.queue_config cf, pgq.active_batch b
      where cf.queue_id = b.queue_id
        and b.batch_id = x_batch_id
        and cf.use_batch_log;
    if found then
        insert into pgq.batch_log (batch_id, queue_id, consumer_id,
            thread_id, tick_start, tick_end, batch_start, batch_end,
            event_count, retry_count, failed_count)
        select b.batch_id, b.queue_id, b.consumer_id,
               b.thread_id, prev.tick_time, cur.tick_time,
               b.creation_date, now(),
               x_total, x_retry, x_failed
            from pgq.active_batch b, pgq.tick cur, pgq.tick prev
            where b.batch_id = x_batch_id
              and cur.tick_id = b.tick_id
              and cur.queue_id = b.queue_id
              and prev.tick_id = b.prev_tick_id
              and prev.queue_id = b.queue_id;
    end if;

    -- remove reservation
    delete from pgq.active_batch
        where batch_id = x_batch_id;
    if not found then
        raise warning 'finish_batch: batch % timed out but thread alive', x_batch_id;
        return 0;
    end if;

    return 1;
end;
$$ language plpgsql security definer;

create or replace function pgq.finish_batch(x_batch_id bigint, x_total integer)
returns integer as $$
declare
    x_retry  integer;
    x_failed integer;
begin
    perform 1 from pgq.queue_config cf, pgq.active_batch b
        where cf.queue_id = b.queue_id
          and b.batch_id = x_batch_id
          and cf.use_batch_log;
    if found then
        select count(*) into x_retry
            from pgq.retry_event where batch_id = x_batch_id;
        select count(*) into x_failed
            from pgq.failed_event where batch_id = x_batch_id;
    end if;
    return pgq.finish_batch(x_batch_id, x_total, x_retry, x_failed);
end;
$$ language plpgsql security definer;

create or replace function pgq.finish_batch(x_batch_id bigint)
returns integer as $$
declare
    x_total  integer;
begin
    perform 1 from pgq.queue_config cf, pgq.active_batch b
        where cf.queue_id = b.queue_id
          and b.batch_id = x_batch_id
          and cf.use_batch_log;
    if found then
        select count(*) into x_total
            from pgq.fetch_batch_events(x_batch_id);
    end if;
    return pgq.finish_batch(x_batch_id, x_total);
end;
$$ language plpgsql security definer;

