/*
--
-- Info about queues
--

create or replace function pgq.get_queue_info()
returns setof pgq.ret_queue_info as $$
declare
    qname text;
    ret   pgq.ret_queue_info%rowtype;
begin
    for qname in
        select queue_name from pgq.queue_config order by 1
    loop
        select * into rec from pgq.get_queue_info(qname);
        return rec;
    return;
end;
$$ language plpgsql security definer;

create or replace function pgq.get_queue_info(qname text)
returns pgq.ret_queue_info as $$
declare
    ret   pgq.ret_queue_info%rowtype;
begin
    select q.queue_name, q.rotation_delay, q.number_of_event_tables,
           -- q.tick_max_amount, q.tick_max_lag, q.tick_idle_interval,
           -- q.external_ticker, q.creation_date,
           NULL,
           (select current_timestamp - t.tick_time
              from pgq.tick t where t.queue_id = q.queue_id
             order by t.queue_id desc, t.tick_id desc limit 1
            ) as ticker_lag
      into ret
      from pgq.queue_config q
      where queue_name = a_queue;
    return ret;
end;
$$ language plpgsql security definer;

create or replace function pgq.get_consumer_info(x_queue_name text)
returns setof pgq.ret_consumer_list as $$
declare
    ret  pgq.ret_consumer_list%rowtype;
begin
    for ret in 
        select q.queue_name, c.consumer_name, wm.thread_timeout,
               current_timestamp - t.tick_time as lag,
               current_timestamp - wm.last_modified_date as last_seen
          from pgq.high_watermark wm, pgq.tick t, pgq.queue_config q,
               pgq.consumer c
         where t.tick_id = wm.last_tick_id
           and q.queue_id = wm.queue_id
           and t.queue_id = wm.queue_id
           and c.consumer_id = wm.consumer_id
           and q.queue_name = x_queue_name
         order by 1,2
    loop
        return next ret;
    end loop;
    return;
end;
$$ language plpgsql security definer;

*/
