
create or replace function pgq.insert_event_raw(
        producer text, queue_name text,
        ref_id text, payload text,
        event_id bigint, creation_date timestamptz,
        consumer_id integer, retry_count integer)
returns bigint as $$

    # load args
    producer = args[0]
    queue_name = args[1]
    ref_id = args[2]
    payload = args[3]
    event_id = args[4]
    creation_date = args[5]
    consumer_id = args[6]
    retry_count = args[7]

    if not "cf_plan" in SD:
        # get current event table
        q = "select table_name, active_event_table, event_seq_name "\
            " from pgq.queue_config"\
            " where queue_name = $1"
        SD["cf_plan"] = plpy.prepare(q, ["text"])

        # get next id
        q = "select nextval($1) as id"
        SD["seq_plan"] = plpy.prepare(q, ["text"])

    # get queue config
    res = plpy.execute(SD["cf_plan"], [queue_name])
    if len(res) != 1:
        plpy.error("Unknown event queue: %s" % (queue_name))
    tbl_prefix = res[0]["table_name"]
    cur_nr = res[0]["active_event_table"]
    id_seq = res[0]["event_seq_name"]

    # get id
    if event_id is None:
        res = plpy.execute(SD['seq_plan'], [id_seq])
        event_id = res[0]["id"]

    # create plan for insertion
    ins_plan = None
    ins_key = "ins.%s" % (queue_name)
    if ins_key in SD:
        nr, ins_plan = SD[ins_key]
        if nr != cur_nr:
            ins_plan = None
    if ins_plan == None:
        q = "insert into %s_%s (event_id, ref_id, payload, producer,"\
            " creation_date, consumer_id, retry_count)"\
            " values ($1, $2, $3, $4, $5, $6, $7)" % (
            tbl_prefix, cur_nr)
        types = ["int8", "text", "text", "text", "timestamptz", "int4", "int4"]
        ins_plan = plpy.prepare(q, types)
        SD[ins_key] = (cur_nr, ins_plan)

    # insert the event
    plpy.execute(ins_plan, [event_id, ref_id, payload, producer,
                            creation_date, consumer_id, retry_count])

    # done
    return event_id

$$ language plpythonu;  -- event inserting needs no special perms

create or replace function pgq.insert_event(
        producer text, queue_name text,
        ref_id text, payload text)
returns bigint as $$
begin
    return pgq.insert_event_raw(producer, queue_name,
            ref_id, payload, null, now(), null, null);
end;
$$ language plpgsql;  -- event inserting needs no special perms

create or replace function pgq.insert_event(queue_name text, ref_id text, payload text)
returns bigint as $$
begin
    return pgq.insert_event(null, queue_name, ref_id, payload);
end;
$$ language plpgsql;  -- event inserting needs no special perms

