create or replace function pgq.maint_retry_events()
returns integer as $$
-- moves retry events back to main queue
-- should be called repeatedly, each call in separate TX
-- until returns 0
declare
    cnt    integer;
    rec    record;
begin
    cnt := 0;
    for rec in
        select pgq.insert_event_raw(
                    ev.producer, cf.queue_name,
                    ev.ref_id, ev.payload, ev.event_id, ev.creation_date,
                    ev.consumer_id, ev.retry_count) as event_id,
               ev.queue_id, ev.consumer_id
          from pgq.retry_event ev, pgq.queue_config cf
         where ev.retry_after_date <= current_timestamp
           and cf.queue_id = ev.queue_id
         order by ev.retry_after_date
         limit 10
    loop
        cnt := cnt + 1;
        delete from pgq.retry_event
         where queue_id = rec.queue_id
           and consumer_id = rec.consumer_id
           and event_id = rec.event_id;
    end loop;
    return cnt;
end;
$$ language plpgsql; -- need admin access

