
create or replace function pgq.stats_for_consumer_in_timespan(
    i_consumer_name text,
    t_start timestamptz,
    t_end timestamptz)
returns setof pgq.ret_stats as $$
declare
    rec pgq.ret_stats%rowtype;
begin
    for rec in 
        select cf.queue_name, co.consumer_name,
            avg(log.batch_end - log.batch_start) as avg_batch_time,
            avg(log.batch_start - log.tick_end) as avg_lag,
            max(log.batch_start - log.tick_end) as max_lag,
            avg(log.event_count) as avg_batch,
            max(log.event_count) as max_batch,
            sum(log.event_count) as total_count,
            sum(log.retry_count) as total_retry,
            sum(log.failed_count) as total_failed,
            -- describe the load
            ( sum(log.event_count) / extract(epoch from (t_end - t_start))
            ) as avg_events_per_sec,
            -- try to take account only actual processing time
            ( sum(log.event_count) / extract(epoch from
                    sum(batch_end - batch_start))
            ) as guess_speed
        from pgq.high_watermark c left join pgq.batch_log log
            on (c.queue_id = log.queue_id
                and c.consumer_id = log.consumer_id
                and log.batch_start >= t_start
                and log.batch_start <= t_end),
            pgq.queue_config cf,
            pgq.consumer co
        where c.consumer_id = co.consumer_id
          and cf.queue_id = c.queue_id
          and co.consumer_name = i_consumer_name
        group by 1, 2, 3
        order by 1, 2, 3
    loop
        return next rec;
    end loop;
    return;
end;
$$ language plpgsql security definer;

create or replace function pgq.stats_for_consumer(
    i_consumer_id text,
    i_time interval)
returns setof pgq.ret_stats as $$
declare
    rec pgq.ret_stats%rowtype;
begin
    for rec in
        select * from pgq.stats_for_consumer_in_timespan(
            i_consumer_id, now() - i_time, now())
    loop
        return next rec;
    end loop;
    return;
end;
$$ language plpgsql;

create or replace function pgq.stats_in_timespan(
    t_start timestamptz, t_end timestamptz)
returns setof pgq.ret_stats as $$
declare
    rec pgq.ret_stats%rowtype;
    cons record;
begin
    for cons in 
        select consumer_name from pgq.high_watermark wm, pgq.consumer co
            where co.consumer_id = wm.consumer_id
            order by 1
    loop
        for rec in
            select * from pgq.stats_for_consumer_in_timespan(
                cons.consumer_name, t_start, t_end)
        loop
            return next rec;
        end loop;
    end loop;
    return;
end;
$$ language plpgsql; -- no perms needed

create or replace function pgq.stats_in_interval(tv interval)
returns setof pgq.ret_stats as $$
declare
    rec pgq.ret_stats%rowtype;
begin
    for rec in
        select * from pgq.stats_in_timespan(now() - tv, now())
    loop
        return next rec;
    end loop;
    return;
end;
$$ language plpgsql; -- no perms needed

-- select avg_events_per_sec from pgq.stats_for_consumer(consumer_id, '5 minutes');


